filters

package
v1.1.1 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Apr 10, 2026 License: MIT Imports: 18 Imported by: 0

Documentation

Overview

Package filters is an internal package that defines the standard Liquid filters.

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func AddStandardFilters

func AddStandardFilters(fd FilterDictionary)

AddStandardFilters defines the standard Liquid filters.

Types

type FilterDictionary

type FilterDictionary interface {
	AddFilter(string, any)
	AddContextFilter(string, expressions.ContextFilterFn)
}

A FilterDictionary holds filters.

type ZeroDivisionError

type ZeroDivisionError struct{}

ZeroDivisionError is returned by the divided_by and modulo filters when the divisor is zero. Use errors.As to detect this specific condition.

func (*ZeroDivisionError) Error

func (e *ZeroDivisionError) Error() string

Jump to